/* *******************************************************************

   Minimum Example for a webserver on the ESP8266 / ESP32

   Hardware
   This sketch supports ESP8266 and ESP32 based boards

   Software
   The Software was last tested with
   - IDE 1.8.19
   - ESP32 Core 2.0.6, 1.0.6    (2.0.8 can have issues)
   - ESP8266 Core 3.0.2, 2.7.4

   Short
   - simple webserver with some pages
   - Stylesheet (css) optimized for mobile devices
   - ArduinoOTA upload with Arduino IDE

   Next
   - see an advanced webserver tutorial on: https://werner.rothschopf.net/

   Open Tasks
   - to be done

   by noiasca

   Version
   2023-07-27 First strip down version based on the generic webserver

***************************************************************** */

#ifdef ARDUINO_ARCH_ESP8266            // libraries if you compile for a ESP8266
#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>          // for the webserver
#include <ESP8266mDNS.h>               // Bonjour/multicast DNS, finds the device on network by name
#include <ArduinoOTA.h>                // OTA Upload via ArduinoIDE
#endif
#ifdef ARDUINO_ARCH_ESP32              // libraries if you compile for a ESP32
#include <WiFi.h>
#include <WiFiClient.h>
#include <WebServer.h>                 // for the webserver
#include <ESPmDNS.h>                   // Bonjour/multicast DNS, finds the device on network by name   
#include <ArduinoOTA.h>                // OTA Upload via ArduinoIDE
#endif

#define VERSION "23.10.0"              // the version of this sketch

/* *******************************************************************
         define which board variant should be compiled
         set the number to one of your defined boards
 ********************************************************************/

#define USE_BOARD 1                             // the actual board to compile:
#define TXT_BOARDNAME "ESP Webserver Maxi"      // the name of the board on webgui and Serial
const uint8_t wifiActivityPin = 2;              // if you want to indicate Wifi activity on a pin. Set to 255 if activity LED is not needed

/* *******************************************************************
         other settings / weitere Einstellungen für den Anwender
 ********************************************************************/

//#include <credentials.h>               // if you have an external file with your credentials you can use it - remove before upload

#ifndef STASSID                        // either use an external .h file containing STASSID and STAPSK - or 
#define STASSID "your-ssid"            // ... modify these line to your SSID
#define STAPSK  "your-password"        // ... and set your WIFI password
#endif

const char* ssid = STASSID;
const char* password = STAPSK;

/* *******************************************************************
         Globals - Variables and constants
 ********************************************************************/

#ifdef ARDUINO_ARCH_ESP8266
ESP8266WebServer server(80);           // an instance for the webserver
#endif
#ifdef ARDUINO_ARCH_ESP32
WebServer server(80);
#endif

/* *******************************************************************
         F U N C T I O N S
 ********************************************************************/

void wifiConnect() {
  if (wifiActivityPin < 255)
    pinMode(wifiActivityPin, OUTPUT);
  char myhostname[42] {"esp"};         // The sketch will add the board ID. Take care about the total length of the char array (including null)
  char val[8];
  itoa(USE_BOARD, val, 10);            // before we can concatenate the precompiler integer, we have to convert it to char array
  strcat(myhostname, val);
  WiFi.hostname(myhostname);
#ifdef ARDUINO_ARCH_ESP32
  WiFi.setSleep(false);                // disable powersafe https://github.com/espressif/arduino-esp32/issues/1484
#endif
  Serial.print(F("try "));
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  // Wait for connection
  int8_t i = 0;  // just a counter to be able generate a line feed
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print('.');
    if (wifiActivityPin < 255)
      i % 2 ? digitalWrite(wifiActivityPin, HIGH) : digitalWrite(wifiActivityPin, LOW); // blink LED as indicator
    i++;
    if (i >= 40) {
      Serial.println();
      i = 0;
    }
  }
  Serial.print(F("\nConnected to "));
  Serial.println(ssid);
  Serial.print(F("IP address: "));
  Serial.println(WiFi.localIP());
  if (MDNS.begin(myhostname)) {
    Serial.println(F("MDNS responder started"));
  }
  if (wifiActivityPin < 255) digitalWrite(wifiActivityPin, HIGH);
}

/* *******************************************************************
         S E T U P
 ********************************************************************/

void setup(void) {
  Serial.begin(115200);
  Serial.print(F("\n" TXT_BOARDNAME "\nVersion: " VERSION " Board "));
  Serial.println(USE_BOARD);
  Serial.print(__DATE__);
  Serial.print(" ");
  Serial.println(__TIME__);

  wifiConnect();
  //each example has it's own "setup" - so you need to call all used server examples 
  server10Setup();  // a simple HTML5 page
  server20Setup();  // a simple page styled with CSS
  server30Setup();  // simple output of variables
  server40Setup();  // entry form for user data
  server50Setup();  // switch pins via webpage
  server55Setup();  // perform actions via webpage
  server70Setup();  // output of variables and update with FetchAPI/JavaScript 

  server.begin();                                // start the webserver
  Serial.println(F("HTTP server started\n"));

  // some examples need an additional call for used HW in setup()
  example50Setup();

  ArduinoOTASetup(); // init IDE OTA
}

/* *******************************************************************
         M A I N L O O P
 ********************************************************************/

void loop(void) {
  server.handleClient();               // call the webserver
  ArduinoOTA.handle();                 // OTA Upload via ArduinoIDE
  // some examples need a HW call in setup()
  example30Loop();
  // do your calculation, sensor readings and update your global variables

  // add displays, serial outputs here
}
//
